@@ -104,10 +104,12 @@ def now_ms():
     return int(time.time() * MILLION)
 
 
-def create_series_work(task=None, callback=None):
-    return pywf.create_series_work(
-        pywf.create_empty_task() if task is None else task, callback
-    )
+def create_series_work(*tasks, callback=None):
+    if not tasks:
+        return pywf.create_series_work(pywf.create_empty_task(), callback)
+    series = pywf.create_series_work(tasks[0], callback)
+    map(series.push_back, tasks[1:])
+    return series
 
 
 def create_timer_task(
